'' =================================================================================================
''
''   File....... jm_countdown_timer.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 18 AUG 2015
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us


con { io pins }

  RX1 = 31                                                       ' programming / terminal
  TX1 = 30

  SDA = 29                                                       ' i2c / eeprom
  SCL = 28


con

  TIME_SET    = 10                                               ' countdown time (1 to 99)

  KEY_SCAN    = 25                                               ' scan keys every 25ms
  KEY_HOLDOFF = 1000                                             ' if key press, wait 1000ms for next check

  #0, S_HOLD, S_RUN                                              ' program states
  

obj

' main                                                           ' * master Spin cog
  time   : "jm_time"                                             '   timing and delays
  ktimer : "jm_time"                                             '   key input timing
  io     : "jm_io_basic"                                         '   essential io control
  term   : "jm_fullduplexserial"                                 ' * serial io for terminal

' * Uses cog when loaded


var

  long  state                                                    ' program state (hold or run)
  long  now                                                      ' current time
  long  last                                                     ' last displayed time (change flag)


pub main 

  setup

  term.rxflush                                                   ' wait open terminal & keypress
  term.rx
  term.tx(term#CLS)

  reset_time                                                     ' reset the timer

  repeat
    check_time                                                   ' check for timer change
    check_keys                                                   ' look for key

    
pub check_keys | k
 
  if (ktimer.millis < 0)                                         ' if not time for key check
    return                                                       '  abort
  else
    ktimer.set(-KEY_SCAN)                                        ' (re)set key scan timing
   
  k := term.rxcheck                                              ' check for key                                           
  if (k => 0)                                                    ' if key pressed
    process_key(k)                                               '  handle it
    term.rxflush                                                 '  clean-up keyboard buffer                                            
                                       

pub process_key(k)

'' Process key from user
'' -- space bar starts/stops timer
'' -- [R] key resets timer when paused

  case k
    " ":
      if (state == S_HOLD)                                       ' on hold?
        state := S_RUN                                           '  back to run
        time.release                                             '  release timer
      elseif (state == S_RUN)                                    ' running?
        state := S_HOLD                                          '  set to hold
        time.hold                                                '  hold the timer
      ktimer.set(-KEY_HOLDOFF)                                   ' extend the hold-off

    "r", "R":                                                    ' reset?
      reset_time                                                 '  yes, do it
      ktimer.set(-KEY_HOLDOFF)


pub reset_time

'' Reset countdown timer
'' -- only allowed in HOLD mode

  if (state == S_HOLD)
    time.hold                                                    ' ensure timer is not running
    time.set_secs(-TIME_SET)                                     ' use negative for count-down
    last := negx                                                 ' preset changed flag


pub check_time | nowms

'' Checks timer for full 1/10s change
'' -- if change, time display is updated
'' -- if time expired, puts program state in HOLD

  nowms := time.millis                                           ' capture time
  now := -nowms / 100                                            ' extract tenths (remove sign)
  if (nowms // 1000)                                             ' if fractional seconds  
    ++now                                                        '  round up for display  

  if (now <> last)                                               ' changed?
    show_time                                                    '  yes, show it
    last := now                                                  '  mark change
    
  if (now == 0)                                                  ' done?
    state := S_HOLD
    time.hold
    
    
pub show_time

'' Display time as 3-digit value in form nn.n 

  term.tx(term#HOME)
  term.tx("0" + (now / 100))                                     ' 10s digit
  term.tx("0" + (now // 100 / 10))                               '  1s digit
  term.tx(".")
  term.tx("0" + (now // 10))                                     ' 1/10ths digit
  

pub setup

'' Setup IO and objects for application

  time.start                                                     ' setup timing/delays for clock speed
  ktimer.start                                                   ' setup key input timer

  io.start(0, 0)                                                 ' clear all pins

  term.start(RX1, TX1, %0000, 115_200)                           ' start serial for terminal


dat { license }

{{

  Copyright (C) 2015 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}